package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.Mandatory;
import com.ether.annotation.ParamName;
import com.ether.annotation.UseJSON;
import com.ether.tapas.Observatory;
import com.ether.tapas.Tapas;
import com.ether.user.User;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * @author vmipsl
 * @date 02 march 2012
 */
public class Controller
        extends ControllerEther
{

    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /** *********************************************************** **/
    /**
     * View for the form
     *
     * @return
     * @throws WebException
     */
    @ControllerMethod(view = VIEW_FORM_TAPAS, loginMandatory = true, defaultView = VIEW_INDEX)
    public JSONObject viewForm()
            throws WebException
    {
        try
        {
            final List<Observatory> allObservatories = getTapasService().getAllObservatories();

            final JSONObject jsonObject = new JSONObject();
            jsonObject.put( "jSonFileFormats", getJSONFileFormat() );
            jsonObject.put( "jSonYesNos", getJSONYesNo() );
            jsonObject.put( "jSonObservatories", allObservatories );
            jsonObject.put( "jSonSpectralUnits", getJSONSpectralUnit() );
            jsonObject.put( "jSonInstrumentalFunctions", getJSONInstrumentalFunction() );
            jsonObject.put( "jSonClimatoReferences", getJSONClimatoReference() );
            return jsonObject;
        }
        catch( ServiceException e )
        {
            throw new WebException( WebException.WebCode.ERROR_NO_OBSERVATORY_FOUND, e );
        }

    }


    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /** *********************************************************** **/
    /**
     * This method create a XML file filled with the javaTapas fields
     *
     * @param javaTapas
     * @param user
     * @return
     * @throws WebException
     */
    @ControllerMethod(jsonResult = true, userMandatory = true, defaultView = VIEW_INDEX)
    //@ControllerMethod(jsonResult = true, userMandatory = false, defaultView = VIEW_INDEX)
    public JSONObject createUserRequest( @ParamName("jsonTapas") @Mandatory @UseJSON final Tapas javaTapas, @NotNull final User user )
            throws WebException
    {

        if( null == javaTapas )
            throw new WebException( WebException.WebCode.ERROR_JSON_TAPAS_IS_NULL, "Serialization error : jsonTapas is null", WebException.getExceptionThrowable() );

        final Properties prop = new Properties();
        final File propertiesFile = new File(WebHelper.PROPERTIES_FILE_PATH + WebHelper.PROPERTIES_FILE);
        try
        {
            final InputStream is = new FileInputStream(propertiesFile);
            prop.load( is );
        }
        catch( FileNotFoundException e )
        {
            throw new WebException( ServiceException.ServiceCode.PROPERTIES_FILE_NOT_FOUND, e );
        }
        catch( IOException e )
        {
            throw new WebException( ServiceException.ServiceCode.PROPERTIES_FILE_NOT_FOUND, e );
        }

        // creation fichier requete xml
        try
        {
            getTapasService().createXMLRequest( javaTapas, prop );
        }
        catch( ServiceException e )
        {
            throw new WebException( WebException.WebCode.ERROR_XML_CREATION, e );
        }

        // creation fichier requete xml
        try
        {
            getTapasService().createUserFtpDir( user, prop );
        }
        catch( ServiceException e )
        {
            throw new WebException( WebException.WebCode.ERROR_USERS_CREATION, e );
        }

        //insertion requete dans bdd
        try
        {
            getTapasService().insertTapasRequest( user );
        }
        catch( ServiceException e )
        {
            throw new WebException( WebException.WebCode.ERROR_INSERT_TAPAS_REQUEST, e );
        }

        //execution du qsub
        try
        {
            getTapasService().execAppelQSub( javaTapas.getId(), user, prop );
        }
        catch( ServiceException e )
        {
            throw new WebException( WebException.WebCode.ERROR_CALL_QSUB, e );
        }


        final JSONObject result = new JSONObject();
        result.put( "result", "Your request will be processed. You will receive the result link by email" );
        return result;
    }

    private List<JSONObject> getJSONFileFormat()
    {
        final FileFormat[] fileFormats = FileFormat.values();

        final List<JSONObject> jsonFileFormats = new ArrayList<JSONObject>( fileFormats.length );

        for( final FileFormat fileFormat : fileFormats )
        {
            final JSONObject jsonFileFormat = new JSONObject();
            jsonFileFormat.put( "text", fileFormat.name() );
            jsonFileFormat.put( "value", fileFormat.name() );
            jsonFileFormats.add( jsonFileFormat );
        }
        return jsonFileFormats;
    }

    private List<JSONObject> getJSONYesNo()
    {
        final YesNo[] yesNos = YesNo.values();

        final List<JSONObject> jsonYesNos = new ArrayList<JSONObject>( yesNos.length );

        for( final YesNo yesNo : yesNos )
        {
            final JSONObject jsonYesNo = new JSONObject();
            jsonYesNo.put( "text", yesNo.name() );
            jsonYesNo.put( "value", yesNo.name() );
            jsonYesNos.add( jsonYesNo );
        }
        return jsonYesNos;
    }

    private List<JSONObject> getJSONSpectralUnit()
    {
        final SpectralUnit[] spectralUnits = SpectralUnit.values();

        final List<JSONObject> jsonSpectralUnits = new ArrayList<JSONObject>( spectralUnits.length );

        for( final SpectralUnit spectralUnit : spectralUnits )
        {
            final JSONObject jsonSpectralUnit = new JSONObject();
            jsonSpectralUnit.put( "text", spectralUnit.name() );
            jsonSpectralUnit.put( "value", spectralUnit.name() );
            jsonSpectralUnits.add( jsonSpectralUnit );
        }
        return jsonSpectralUnits;
    }


    private List<JSONObject> getJSONInstrumentalFunction()
    {
        final InstrumentalFunction[] instrumentalFunctions = InstrumentalFunction.values();

        final List<JSONObject> jsonInstrumentalFunctions = new ArrayList<JSONObject>( instrumentalFunctions.length );
        final JSONObject jsonInstrumentalFunctionNone = new JSONObject();
        jsonInstrumentalFunctionNone.put( "text", InstrumentalFunction.NONE );
        jsonInstrumentalFunctionNone.put( "value", -1 );
        jsonInstrumentalFunctions.add( jsonInstrumentalFunctionNone );

        final JSONObject jsonInstrumentalFunctionGaussian = new JSONObject();
        jsonInstrumentalFunctionGaussian.put( "text", InstrumentalFunction.GAUSSIAN );
        jsonInstrumentalFunctionGaussian.put( "value", 1 );
        jsonInstrumentalFunctions.add( jsonInstrumentalFunctionGaussian );

        return jsonInstrumentalFunctions;
    }


    private List<JSONObject> getJSONClimatoReference()
    {
        final ClimatoReference[] climatoReferences = ClimatoReference.values();

        final List<JSONObject> jsonClimatoReferences = new ArrayList<JSONObject>( climatoReferences.length );

        for( final ClimatoReference climatoReference : climatoReferences )
        {
            final JSONObject jsonClimatoReference = new JSONObject();
            jsonClimatoReference.put( "text", climatoReference.name() );
            jsonClimatoReference.put( "value", climatoReference.ordinal() );
            jsonClimatoReferences.add( jsonClimatoReference );
        }
        return jsonClimatoReferences;
    }


    private static final Log LOGGER = LogFactory.getLog( Controller.class );

    private static final String VIEW_INDEX = "init_en";
    private static final String VIEW_FORM_TAPAS = "project/formTapas";
}